grammar XText;

@lexer::header {
package org.openarchitectureware.xtext.parser.antlr; 

import java.util.ArrayList;
import java.util.List;

import org.antlr.runtime.CharStream;
import org.antlr.runtime.EarlyExitException;
import org.antlr.runtime.Lexer;
import org.antlr.runtime.MismatchedSetException;
import org.antlr.runtime.NoViableAltException;
import org.antlr.runtime.RecognitionException;
import org.openarchitectureware.xtext.parser.ErrorMsg;
import org.openarchitectureware.xtext.parser.impl.AntlrUtil;
}

@parser::header {
package org.openarchitectureware.xtext.parser.antlr; 

import java.util.ArrayList;
import java.util.List;

import org.antlr.runtime.BitSet;
import org.antlr.runtime.CommonToken;
import org.antlr.runtime.NoViableAltException;
import org.antlr.runtime.Parser;
import org.antlr.runtime.RecognitionException;
import org.antlr.runtime.Token;
import org.antlr.runtime.TokenStream;
import org.eclipse.emf.ecore.EObject;
import org.openarchitectureware.xtext.MetaModelRegistration;
import org.openarchitectureware.xtext.XtextFile;
import org.openarchitectureware.xtext.parser.ErrorMsg;
import org.openarchitectureware.xtext.parser.impl.AntlrUtil;
import org.openarchitectureware.xtext.parser.impl.EcoreModelFactory;
import org.openarchitectureware.xtext.parser.model.ParseTreeManager;
import org.openarchitectureware.xtext.parser.parsetree.Node;
}

@lexer::members {
   private List<ErrorMsg> errors = new ArrayList<ErrorMsg>();
	public List<ErrorMsg> getErrors() {
		return errors;
	}
	
	public String getErrorMessage(RecognitionException e, String[] tokenNames) { 
		String msg = super.getErrorMessage(e,tokenNames);
		errors.add(AntlrUtil.create(msg,e,tokenNames));
		return msg; 
	} 
}

@parser::members {

   	private Token getLastToken() {
		return input.LT(-1);
	}
	private Token getNextToken() {
		return input.LT(1);
	}
	
	private int line() {
		Token t = getNextToken();
		if (t==null)
			return 1;
		return t.getLine();
	}
	
	private int start() {
		Token t = getNextToken();
		if (t==null)
			return 0;
		if (t instanceof CommonToken) {
			return ((CommonToken)t).getStartIndex();
		}
		return t.getTokenIndex();
	}
	
	private int end() {
		Token t = getLastToken();
		if (t==null)
			return 1;
		if (t instanceof CommonToken) {
			return ((CommonToken)t).getStopIndex()+1;
		}
		return t.getTokenIndex();
	}
	
	protected Object convert(Object arg) {
		if (arg instanceof org.antlr.runtime.Token) {
			Token t = (Token) arg;
			String s = t.getText();
			if (t.getType() == XTextLexer.ID && s.startsWith("^")) {
				return s.substring(1);
			} else if (t.getType()==XTextLexer.STRING) {
				return s.substring(1,s.length()-1);
			}
			return s;
		}
		return arg;
	}
	
	
	private EcoreModelFactory f= new EcoreModelFactory(MetaModelRegistration.getEPackage());
    private ParseTreeManager ptm = new ParseTreeManager();
	private XtextFile xtextfile = MetaModelRegistration.getXtextFile();
	
	public ParseTreeManager getResult() {
		return ptm;
	}
	
	private List<ErrorMsg> errors = new ArrayList<ErrorMsg>();
	public List<ErrorMsg> getErrors() {
		return errors;
	}
	
	@Override
    public void reportError(RecognitionException e) {
		String msg = super.getErrorMessage(e,tokenNames);
		errors.add(AntlrUtil.create(msg,e,tokenNames));
    	ptm.addError(msg, e);
    	ptm.ruleFinished(null, end());
    }
}

parse returns [Node r] :
  result=xtextFile EOF
  {ptm.ruleFinished(result,end());$r = ptm.getCurrent();};

xtextFile returns [EObject result] : 
    { $result = f.create("XtextFile"); 
       ptm.setModelElement($result);
     }  
    ((({ptm.invokeRule(null,line(),start());}'grammar_name'{ptm.ruleFinished(getLastToken(),end());} name=ID {f.set($result, "name", convert(name));}{ptm.ruleFinished(getLastToken(),end());})?
     ({ptm.invokeRule(null,line(),start());}'grammar_nsURI'{ptm.ruleFinished(getLastToken(),end());} {ptm.invokeRule(null,line(),start());}nsuri=STRING {f.set($result, "nsURI", convert(nsuri));}{ptm.ruleFinished(getLastToken(),end());})?)| 
     ({ptm.invokeRule(null,line(),start());}preventMMGeneration='preventMMGeneration' {f.set($result, "preventMMGeneration", true);}{ptm.ruleFinished(getLastToken(),end());})
    )
    ({ptm.invokeRule(null,line(),start());}commentsflag='commentsDisabled' {f.set($result, "commentsDisabledFlag", true);}{ptm.ruleFinished(getLastToken(),end());})?
    ({ptm.invokeRule(null,line(),start());}i=importGrammar {f.add($result,"imports",convert(i));}{ptm.ruleFinished(i,end());})*
    ({ptm.invokeRule(null,line(),start());}iMM=importMetamodel {f.add($result,"mmImports",convert(iMM));}{ptm.ruleFinished(iMM,end());})*    
    ({ptm.invokeRule(null,line(),start());}r=rule {f.add($result,"rules",convert(r));}{ptm.ruleFinished(r,end());})*
;

importGrammar returns  [EObject result] : 
    {$result = f.create("Import"); 
       ptm.setModelElement($result);
    } 
    		{ptm.invokeRule(null,line(),start());}'importGrammar'{ptm.ruleFinished(getLastToken(),end());}    
		{ptm.invokeRule(null,line(),start());}location=STRING {f.set($result,"location",convert(location));}{ptm.ruleFinished(getLastToken(),end());}
;


importMetamodel returns  [EObject result] : 
    {$result = f.create("MetamodelImport"); 
       ptm.setModelElement($result);
    } 
    		{ptm.invokeRule(null,line(),start());}'importMetamodel'{ptm.ruleFinished(getLastToken(),end());}    
		{ptm.invokeRule(null,line(),start());}location=STRING {f.set($result,"location",convert(location));}{ptm.ruleFinished(getLastToken(),end());}
		({ptm.invokeRule(null,line(),start());}'as'{ptm.ruleFinished(getLastToken(),end());}
   		 {ptm.invokeRule(null,line(),start());}alias=ID {f.set($result,"alias",convert(alias));}{ptm.ruleFinished(getLastToken(),end());})?
   		{ptm.invokeRule(null,line(),start());}';'{ptm.ruleFinished(getLastToken(),end());}
;

rule returns [EObject result] : 
    	a=typeRule {$result=a;}
    |	b=stringRule {$result=b;}
    |	x=lexerRule {$result=x;}
    |	c=enumRule {$result=c;}
;

lexerRule returns [EObject result] :
   {$result = f.create("NativeLexerRule"); 
       ptm.setModelElement($result);
     }  
		{ptm.invokeRule(null,line(),start());}'Native'{ptm.ruleFinished(getLastToken(),end());} 
		{ptm.invokeRule(null,line(),start());}name=ID {f.set($result,"name",convert(name));}{ptm.ruleFinished(getLastToken(),end());}
		{ptm.invokeRule(null,line(),start());}':' {ptm.ruleFinished(getLastToken(),end());}
		{ptm.invokeRule(null,line(),start());}impl=STRING {f.set($result,"impl",convert(impl));} {ptm.ruleFinished(getLastToken(),end());}
		{ptm.invokeRule(null,line(),start());}';'{ptm.ruleFinished(getLastToken(),end());}			
;

typeRule returns [EObject result] : 
    {$result = f.create("TypeRule"); 
       ptm.setModelElement($result);
     } 
    {ptm.invokeRule(null,line(),start());}name=ID {f.set($result,"name",convert(name));}{ptm.ruleFinished(getLastToken(),end());}
    ({ptm.invokeRule(null,line(),start());}'[' {ptm.ruleFinished(getLastToken(),end());}
     {ptm.invokeRule(null,line(),start());}type=typeNameRule {f.set($result,"type",convert(type));}{ptm.ruleFinished(getLastToken(),end());}
     {ptm.invokeRule(null,line(),start());}']' {ptm.ruleFinished(getLastToken(),end());})?
    {ptm.invokeRule(null,line(),start());}':' {ptm.ruleFinished(getLastToken(),end());}
    {ptm.invokeRule(null,line(),start());}content=alternatives  {f.set($result,"content",convert(content));} {ptm.ruleFinished(content,end());}
    {ptm.invokeRule(null,line(),start());}';' {ptm.ruleFinished(getLastToken(),end());}
;


typeNameRule returns [EObject result] :
    {$result = f.create("TypeName"); 
       ptm.setModelElement($result);
     } 
    ({ptm.invokeRule(null,line(),start());}alias=ID {f.set($result,"alias",convert(alias));}{ptm.ruleFinished(getLastToken(),end());}
     {ptm.invokeRule(null,line(),start());}'::' {ptm.ruleFinished(getLastToken(),end());})?
    {ptm.invokeRule(null,line(),start());}name=ID {f.set($result,"name",convert(name));}{ptm.ruleFinished(getLastToken(),end());}
;


stringRule returns [EObject result] : 
    {$result = f.create("StringRule"); 
       ptm.setModelElement($result);
     }
    {ptm.invokeRule(null,line(),start());}'String' {ptm.ruleFinished(getLastToken(),end());}
    {ptm.invokeRule(null,line(),start());}name=ID {f.set($result,"name",convert(name));} {ptm.ruleFinished(getLastToken(),end());}
    {ptm.invokeRule(null,line(),start());}':' {ptm.ruleFinished(getLastToken(),end());}
    {ptm.invokeRule(null,line(),start());}content=alternatives {f.set($result,"content",convert(content));}{ptm.ruleFinished(content,end());}
    {ptm.invokeRule(null,line(),start());}';'{ptm.ruleFinished(getLastToken(),end());}
;

enumRule returns [EObject result] : 
    { $result = f.create("EnumRule"); 
       ptm.setModelElement($result);
     } 
    {ptm.invokeRule(null,line(),start());}'Enum' {ptm.ruleFinished(getLastToken(),end());}
    {ptm.invokeRule(null,line(),start());}name=ID {f.set($result,"name",convert(name));} {ptm.ruleFinished(getLastToken(),end());}
    ({ptm.invokeRule(null,line(),start());}'[' {ptm.ruleFinished(getLastToken(),end());}
     {ptm.invokeRule(null,line(),start());}type=typeNameRule {f.set($result,"type",convert(type));}{ptm.ruleFinished(getLastToken(),end());}
     {ptm.invokeRule(null,line(),start());}']' {ptm.ruleFinished(getLastToken(),end());})?
    {ptm.invokeRule(null,line(),start());}':' {ptm.ruleFinished(getLastToken(),end());}
    {ptm.invokeRule(null,line(),start());}lit=enumLiteral{f.add($result,"literals",convert(lit));} {ptm.ruleFinished(lit,end());}
    ({ptm.invokeRule(null,line(),start());}'|' {ptm.ruleFinished(getLastToken(),end());}
     {ptm.invokeRule(null,line(),start());}lit1=enumLiteral{f.add($result,"literals",convert(lit1));}{ptm.ruleFinished(lit1,end());})* 
    {ptm.invokeRule(null,line(),start());}';'{ptm.ruleFinished(getLastToken(),end());}
;

enumLiteral returns [EObject result] : 
    { $result = f.create("EnumLiteral"); 
       ptm.setModelElement($result);
     }
    {ptm.invokeRule(null,line(),start());}name=ID {f.set($result,"name",convert(name));}{ptm.ruleFinished(getLastToken(),end());}
    {ptm.invokeRule(null,line(),start());}'=' {ptm.ruleFinished(getLastToken(),end());}
    {ptm.invokeRule(null,line(),start());}kw=STRING {f.set($result,"keyword",convert(kw));}{ptm.ruleFinished(getLastToken(),end());}
;

alternatives returns [EObject r] :
   {$r = f.create("Alternatives"); 
       ptm.setModelElement($r);
     }
    {ptm.invokeRule(null,line(),start());}s2=group {f.add($r,"alternatives",convert(s2));}{ptm.ruleFinished(s2,end());}
    ({ptm.invokeRule(null,line(),start());}'|' {ptm.ruleFinished(getLastToken(),end());}
     {ptm.invokeRule(null,line(),start());}s2=group {f.add($r,"alternatives",convert(s2));}{ptm.ruleFinished(s2,end());})*
;

group returns [EObject r] :
   {$r = f.create("Group"); 
    ptm.setModelElement($r);
   }
      {ptm.invokeRule(null,line(),start());}x2=terminal {f.add($r,"children",convert(x2));} {ptm.ruleFinished(x2,end());}
      ({ptm.invokeRule(null,line(),start());}x2=terminal {f.add($r,"children",convert(x2));} {ptm.ruleFinished(x2,end());})*
;

terminal returns [EObject r] :
       (ID ('='|'?='|'+='))=> x1=assignment  {$r=x1;} |
	x2=abstractToken  {$r=x2;};

assignment returns [EObject r]:
   {$r = f.create("Assignment"); 
       ptm.setModelElement($r);
     }
   {ptm.invokeRule(null,line(),start());}feature=ID {f.set($r,"feature",convert(feature));} {ptm.ruleFinished(getLastToken(),end());}
   {ptm.invokeRule(null,line(),start());}op=assignOperator {f.set($r,"operator",convert(op));} {ptm.ruleFinished(op,end());}
   {ptm.invokeRule(null,line(),start());}tok=abstractToken {f.set($r,"token",convert(tok));}{ptm.ruleFinished(tok,end());};


abstractToken returns [EObject r] : 
   (x1=keyword {$r=x1;}|
   x2=ruleName {$r=x2;}|
   x3=reference {$r=x3;}|
   x4=fileReference {$r=x4;} |
   x5=paranthesizedElement {$r=x5;})
   ({ptm.invokeRule(null,line(),start());}ct=cardinalityType {f.set($r,"cardinality",convert(ct));}{ptm.ruleFinished(ct,end());})?
;

paranthesizedElement returns [EObject r] :
   {ptm.invokeRule(null,line(),start());}'(' {ptm.ruleFinished(getLastToken(),end());}
   {ptm.invokeRule(null,line(),start());}x2=alternatives {$r=x2;ptm.ruleFinished(x2,end());}
   {ptm.invokeRule(null,line(),start());}')'{ptm.ruleFinished(getLastToken(),end());}
;

fileReference returns [EObject r] :
     {$r=f.create("FileRef");
       ptm.setModelElement($r);
     }
     {ptm.invokeRule(null,line(),start());} 'URI' {ptm.ruleFinished(getLastToken(),end());}
;

keyword returns [EObject r] : 
   {$r=f.create("Keyword");
       ptm.setModelElement($r);
     }
   {ptm.invokeRule(null,line(),start());}s=STRING { f.set($r,"value",convert(s));}{ptm.ruleFinished(getLastToken(),end());}
;

ruleName returns [EObject r] :
   {$r=f.create("RuleName"); 
       ptm.setModelElement($r);
     }
   {ptm.invokeRule(null,line(),start());}id=ID {f.set($r,"name",convert(id));}{ptm.ruleFinished(getLastToken(),end());};

reference returns [EObject r] :
   {$r=f.create("CrossReference"); 
       ptm.setModelElement($r);
     }
   {ptm.invokeRule(null,line(),start());}'[' {ptm.ruleFinished(getLastToken(),end());}
   {ptm.invokeRule(null,line(),start());} type=typeNameRule {f.set($r,"type",convert(type));}{ptm.ruleFinished(getLastToken(),end());}
   (
   {ptm.invokeRule(null,line(),start());}'|' {ptm.ruleFinished(getLastToken(),end());}
   {ptm.invokeRule(null,line(),start());} temp_ruleName=ruleName {f.set($r,"ruleName",convert(temp_ruleName));}{ptm.ruleFinished(getLastToken(),end());}
   )?
   {ptm.invokeRule(null,line(),start());}']' {ptm.ruleFinished(getLastToken(),end());};

assignOperator returns [Object e] :
     '=' {e=f.enumLit("AssignOperator","ASSIGN");}
   | '?=' {e=f.enumLit("AssignOperator","BOOLASSIGN");}
   | '+=' {e=f.enumLit("AssignOperator","ADD");}
;

cardinalityType returns [Object e] :
     '?' {e=f.enumLit("CardinalityType","OPTIONAL");}
   | '*' {e=f.enumLit("CardinalityType","ANY");}
   | '+' {e=f.enumLit("CardinalityType","ONEORMORE");}
;


STRING :  
    '"' ( '\\' ('b'|'t'|'n'|'f'|'r'|'\"'|'\''|'\\') | ~('\\'|'"') )* '"' |
    '\'' ( '\\' ('b'|'t'|'n'|'f'|'r'|'\"'|'\''|'\\') | ~('\\'|'\'') )* '\'';

ID	:
	('^')?('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')*
;

XTEXT_INT :
	('0'..'9')+ 
;

XTEXT_ML_COMMENT
    :   '/*' ( options {greedy=false;} : . )* '*/' {$channel=HIDDEN;}
    ;

XTEXT_SL_COMMENT
    : '//' ~('\n'|'\r')* '\r'? '\n' {$channel=HIDDEN;}
    ;

XTEXT_WS : (' '|'\t'|'\r'|'\n')+ {$channel=HIDDEN;} ; 
