/*******************************************************************************
 * Copyright (c) 2007 Sven Efftinge (http://www.efftinge.de) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     committers of openArchitectureWare - initial API and implementation
 *******************************************************************************/

package org.openarchitectureware.xtext;

import java.net.URL;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.openarchitectureware.workflow.util.ResourceLoader;
import org.openarchitectureware.workflow.util.ResourceLoaderFactory;
import org.openarchitectureware.xtext.parser.impl.EclipseResourceLoaderImpl;

public class MetaModelRegistration {

	private static Log log = LogFactory.getLog(MetaModelRegistration.class);

	public MetaModelRegistration() {
	}

	static {
		loadXtextFile();
	}

	public static EPackage getEPackage() {
		return XtextPackage.eINSTANCE;
	}

	public static XtextFile xtextfile;

	public static XtextFile getXtextFile() {
		return xtextfile;
	}

	private static void loadXtextFile() {
		if (xtextfile == null) {
			ResourceLoader rl = ResourceLoaderFactory.createResourceLoader();
			try {
				// intialize xtext mm
				org.openarchitectureware.xtext.XtextPackage.eINSTANCE.getEFactoryInstance();
				ResourceLoaderFactory.setCurrentThreadResourceLoader(new EclipseResourceLoaderImpl(
						MetaModelRegistration.class.getClassLoader()));

				final URL url = ResourceLoaderFactory.createResourceLoader().getResource(
						"org/openarchitectureware/xtext/xtext.xmi");

				// Get the URI of the model file.
				URI fileURI = URI.createURI(url.toExternalForm());

				ResourceSet resourceSet = new ResourceSetImpl();
				resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
						Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());

				// FIXME Instead of generally ignoring all errors, a more
				// sophisticated error checking should be implemented.
				resourceSet.getLoadOptions().put(XMLResource.OPTION_RECORD_UNKNOWN_FEATURE, Boolean.TRUE);
				Resource resource = resourceSet.getResource(fileURI, true);

				resource.load(null);
				xtextfile = (XtextFile) resource.getContents().get(0);
			}
			catch (Exception e) {
				log.error("Couldn't load xmi file (org/openarchitectureware/xtext/xtext.xmi)");
			}
			finally {
				ResourceLoaderFactory.setCurrentThreadResourceLoader(rl);
			}
		}
	}
}
